
#include "Palettes.h"

/* set all LEDs of the Strip */
void setLEDs(int r, int g, int b) {
  for (int i = 0; i < NUM_LEDS_PER_STRIP; i++) {
    //leds.setPixelColor(i, r, g, b);
    strips[i] = CRGB(r, g, b);
  }
}

/* set all LEDs of the Strip */
void set2ndLEDs(int r, int g, int b) {
  for (int i = 0; i < NUM_LEDS_PER_STRIP; i++) {
    if (i % 2 == 0)
      fastLEDs[i] = CRGB(r, g, b);
    else
      fastLEDs[i] = CRGB(0, 0, 0);
  }

  mirrorStrips();
  FastLED.show();
}



/* Fade in LEDs at beginning of the Strip */
void fadeBeginning(int numLEDs) {
  for ( int i = 0; i < numLEDs; i++) {
    int brt = i * 255 / numLEDs;
    strips[i].nscale8( brt);
  }
}
/* Fade in LEDs at beginning of the Strip */
void fadeEnd(int numLEDs) {
  for ( int i = 0; i < numLEDs; i++) {
    int index = NUM_LEDS_PER_STRIP - i;
    int brt = map(i, 0, numLEDs, 0, 255);
    strips[index].nscale8( brt);
  }
}

/* Mirror-Copy first half of strip to second half */
void mirrorStrips() {
  for (int i = 0; i < NUM_LEDS_PER_STRIP; i++) {
    //int backIndex = NUM_LEDS_PER_STRIP - i - 1;
    //strips[backIndex] = strips[i];

    fastLEDs[i] = CRGB(strips[i].r, strips[i].g, strips[i].b);
    //fastLEDs[backIndex] = CRGB(strips[i].r, strips[i].g, strips[i].b);
    fastLEDsBack[i] = CRGB(strips[i].r, strips[i].g, strips[i].b);
  }
}

void clearStrip() {
  for (int i = 0; i < 32; i++) {
    strips[i] = CRGB(0, 0, 0);
  }
  mirrorStrips();
}


long lmin(long a, long b) {
  if (a < b)
    return a;
  else
    return b;
}
long lmax(long a, long b) {
  if (a > b)
    return a;
  else
    return b;
}

long mapC(long x, long in_min, long in_max, long out_min, long out_max)
{
  long result = map(x, in_min, in_max, out_min, out_max);
  return constrain(result, lmin(out_min, out_max), lmax(out_min, out_max));
}


void WiFiOn() { //not in use
  wifi_fpm_do_wakeup();
  wifi_fpm_close();
  wifi_set_opmode(STATION_MODE);
  wifi_station_connect();
}


void WiFiOff() {  //not in use
  wifi_station_disconnect();
  wifi_set_opmode(NULL_MODE);
  wifi_set_sleep_type(MODEM_SLEEP_T);
  wifi_fpm_open();
  wifi_fpm_do_sleep(FPM_SLEEP_MAX_TIME);
}

int whichAnimation() { //returns 1 for normal, 2 for "Thermometer"
  //Animation just from sek 0-9, 20-29, 40-49 und in the beginning -> || startWithAnimation
  if (timeClient.getSeconds() <= 9 || (timeClient.getSeconds() >= 20 && timeClient.getSeconds() <= 29) ||  (timeClient.getSeconds() >= 40 && timeClient.getSeconds() <= 49) || startWithAnimation) {
    if (timeClient.getSeconds() == 5 || timeClient.getSeconds() == 25 || timeClient.getSeconds() == 45) startWithAnimation = false;
    currentLED = 0;
    return 1;
  } else {
    return 2;
  }
}

void pulseForLEDs(int maxPulse) { //modifies "counter", that it counts to "maxPulse" and back
  if (counter >= maxPulse) {
    counter = maxPulse;
    currentLED = 1; //global var to play -> change direction
  } else if (counter <= 2) {
    counter = 2;
    currentLED = 0;
  }
  if (currentLED == 1) counter = counter - 2; //-2 -> in LEDUpdate counter ++
}

void LedDot(int pos, int radius, int r, int g, int b) { //draws point which fades out to the edge
  if(radius < 0) return;
  for (int i = 0; i <= radius; i ++) {
    int posVorne = pos + i;
    int posHinten = pos - i;
    if(posVorne >= 0 && posVorne < NUM_LEDS_PER_STRIP) strips[posVorne] = CRGB(r-((r/(radius+1))*i), g-((g/(radius+1))*i), b-((b/(radius+1))*i));
    if(posHinten >= 0 && posHinten < NUM_LEDS_PER_STRIP) strips[posHinten] = CRGB(r-((r/(radius+1))*i), g-((g/(radius+1))*i), b-((b/(radius+1))*i));
  }
}
