
//Sketch for Stadtluft-Anzeiger Home v2.1

//Board: Generic ESP8266 Module, 4M (2M Spiffs)

#include <FS.h> //this needs to be first, or it all crashes and burns... - custom Wifi Manager
#include <LittleFS.h>

//#define FASTLED_ESP8266_RAW_PIN_ORDER //only with NodeMcu for Prototyping

#include <ArduinoOTA.h>
#include <WiFiManager.h>

#include <ESP8266WiFi.h>
#include <WiFiClientSecure.h>
#include <ESP8266HTTPClient.h>

#define FASTLED_ALLOW_INTERRUPTS 0
#include<FastLED.h>
//#include<Metro.h>
#include <Ticker.h>

#include <ArduinoJson.h>        //custom Wifi Manager

//NTP
#include <WiFiUdp.h>
#include <NTPClient.h>

WiFiUDP ntpUDP;
NTPClient timeClient(ntpUDP, "europe.pool.ntp.org", 3600, 60000);


extern "C" {
#include "user_interface.h"  // Required for wifi_station_connect() to work
}
#define FPM_SLEEP_MAX_TIME 0xFFFFFFF

#include "debug.h"

//custom Wifi Manager
char brightness[4] = "40";
int brightnessLevel = 1;
int brightnessSteps[5] = {10, 40, 80, 160, 255};
bool oldButtonState = false;

//flag for saving data
bool shouldSaveConfig = false;

//Put a own name here to personalize / identify your Stadtluft-Anzeiger 
const char apName[40]  = "Stadtluftanzeiger_Jakob";
const char OTAName[40] = "Stadtluftanzeiger_Jakob";

char station[50] = "136-ringkirche"; //for custom Wifi Manager
const int httpsPort = 443;
char host[20] = "nox.s-v.io";
char apiURL[40]  = "/LEDStickMini.php?stationid=";

// Use WiFiClientSecure class to create TLS connection
//just to fake a https request -> but its http -> simpler and without certificates 
WiFiClientSecure httpsclient;
HTTPClient httpclient;
int failcount = 0; 
int maxConsecutiveFails = 3; //reset if the ESP failes x times to connect

const int NUM_LEDS_PER_STRIP = 32;
const int FPS                = 60;
float     no2Level = -1;
float     oldLevel = 100;
int       levelRed = 40;

#define PIN 13    //old Layout: 14, new Layout: front 13, back 12
#define PINBACK 12

#define BUTTON_PIN 14 //old Layout: 12, new Layout: 14

CRGB strips[NUM_LEDS_PER_STRIP];
CRGB fastLEDs[NUM_LEDS_PER_STRIP];
CRGB fastLEDsBack[NUM_LEDS_PER_STRIP];

Ticker ledTicker;

static long counter = 0;
boolean startWithAnimation = true;
int currentLED = 0;

int currentStatus = 1; //0 = Error, 1 = Startup, 2 = Config Portal, 3 = now Values, 4 = normal Animation, 5 = "thermometer" Animation
int oldStatus = 1;


void setup() {

  ESP.wdtDisable(); //turnes off Software Watchdog, because sometimes getData() takes some time
  ESP.wdtEnable(3000); //enables Software Watchdog -> reset after 3sek with no Watchdog response


#if SERIAL_DEBUG
  Serial.begin(115200);
#endif

  pinMode(BUTTON_PIN, INPUT);

  FastLED.addLeds<NEOPIXEL, PIN>(fastLEDs, NUM_LEDS_PER_STRIP);
  FastLED.addLeds<NEOPIXEL, PINBACK>(fastLEDsBack, NUM_LEDS_PER_STRIP);

  FastLED.setBrightness(100); //not in use any more

  ledTicker.attach(1.0 / FPS, ledUpdate);

  connectToWifi(apName);

  initOTA();
  timeClient.begin();

  getData();
}


void loop() {

  timeClient.update();
  ArduinoOTA.handle();

  if (timeClient.getSeconds() == 0 && currentStatus >= 3) {
    getData();
    debugln("Zeit: " +   timeClient.getFormattedTime());
  }

  if (digitalRead(BUTTON_PIN) == HIGH && digitalRead(BUTTON_PIN) != oldButtonState) {
    brightnessLevel++;
    if (brightnessLevel >= 5) brightnessLevel = 0;
    FastLED.setBrightness(brightnessSteps[brightnessLevel]);
    String(brightnessSteps[brightnessLevel]).toCharArray(brightness, 4);
    Serial.println("saving brightness" + String(brightnessSteps[brightnessLevel]));
    saveConfig();
  }
  oldButtonState = digitalRead(BUTTON_PIN);
}


void ledUpdate() {  //exectues FPS / sek -> controls animations
  if (currentStatus == 2 && millis() > 180000) {
    ESP.reset();
  }

  if (no2Level >= 0 && currentStatus == 4 || no2Level >= 0 && currentStatus == 5) {   //which Animation im normal use -> time
    if (whichAnimation() == 1) {
      currentStatus = 4;
    } else if (whichAnimation() == 2) {
      currentStatus = 5;
    }
  }

  if (currentStatus != oldStatus) {
    counter = 0;
    oldStatus = currentStatus;
  }

  switch (currentStatus) {  //choose right Animation
    case 1:     //bootup animation
      animateStartup();
      break;
    case 2:     //config animation
      animateConfig();
      break;
    case 3:     //no Value animation
      animateKeinWert();
      break;
    case 4:     //normale Animation
      if (no2Level < levelRed) {
        animateGreen(counter, 1, 5, 16);
      } else {
        animateRed(counter, 1, 5, 16);
      }
      break;
    case 5:     //Thermometer Animation
      temperaturemeterAnimation();
      break;
    default:    //Error Animation
      animateError();
      break;
  }

  if (counter % 50 == 0) debug(".");

  mirrorStrips();
  FastLED.show();

  counter++;
}
